/*
 	File:		dcmd.h
 
 	Contains:	MacsBug debugger command interface.
 
 	Version:	Technology:	MacsBug
 
 				Release:	6.5.3
 
 	Copyright:	 1984-1995 by Apple Computer, Inc.
 
 				All rights reserved.
 
 	Bugs?:		If you find a problem with this file, send the file and version
 				information (from above) and the problem description to:
 
 					Internet:	apple.bugs@applelink.apple.com
 					AppleLink:	APPLE.BUGS
 
*/
#ifndef __DCMD__
#define __DCMD__

#ifndef __TYPES__
#include <Types.h>
#endif
#ifndef __MACHINEEXCEPTIONS__
#include <MachineExceptions.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=mac68k
#endif

#if PRAGMA_IMPORT_SUPPORTED
#pragma import on
#endif

/* Possible requests from the debugger to the command*/

enum {
	dcmdInit					= 0,							/* Initialize the Dcmd */
	dcmdDoIt					= 1,							/* Normal Dcmd execution */
	dcmdHelp					= 2,							/* Display help for Dcmd */
																/* Requests added to MacsBug in 6.5d10 that are only sent to format 3 or newer dcmds. */
	dcmdSecondaryInit			= 3,							/* Second time to init after all System patches have been loaded */
	dcmdShutdown				= 4,							/* Dcmd should remove any patches (if possible) */
	dcmdGetInfo					= 5								/* Return dcmd version and pointer to help text */
};

/* 68K register file indicesinto the RegisterFile.*/

enum {
	D0Register					= 0,
	D1Register					= 1,
	D2Register					= 2,
	D3Register					= 3,
	D4Register					= 4,
	D5Register					= 5,
	D6Register					= 6,
	D7Register					= 7,
	A0Register					= 8,
	A1Register					= 9,
	A2Register					= 10,
	A3Register					= 11,
	A4Register					= 12,
	A5Register					= 13,
	A6Register					= 14,
	A7Register					= 15,
	PCRegister					= 16,
	SRRegister					= 17							/* SR is only 16 bits and is stored in the high word */
};

/* Heap block types*/

enum {
	freeBlock					= 0,
	nonrelocatableBlock			= 1,
	relocatableBlock			= 2
};

/* Structure used to pass information to and from the dcmd.*/
struct dcmdBlock {
	long *							registerFile;				/* pointer to 68K CPU register set */
	short 							request;					/* what action we are requested to take */
	Boolean 						aborted;					/* Set to true if the user types a key while scrolling */
	Boolean 						pad1;						/* align to word boundary */
	UInt32 							macsBugVersion;				/* version of MacsBug we are running under */
	UInt16 							maxCallback;				/* maximum valid callback we can make */
	UInt8 							currentISA;					/* ISA of current PC */
	UInt8 							pad2;						/* align to word boundary */
	ExceptionInformationPowerPC *	theException;				/* Pointer to PowerPC machine state if ISA is PowerPC */
	Ptr 							requestIOBlock;				/* general-purpose input/output data block pointer */
};
typedef struct dcmdBlock dcmdBlock;

typedef struct dcmdBlock *dcmdBlockPtr;

struct GetInfoRequestBlock {
	Str255 							usageStr;
	Str255 							creditsStr;
	NumVersion 						dcmdVersion;
};
typedef struct GetInfoRequestBlock GetInfoRequestBlock;

typedef struct GetInfoRequestBlock *GetInfoRequestBlockPtr;

/*
	These are some useful macros for filling in format 3 version and string fields.
*/

#define dcmdFillVersion(paramPtr, version) \
	* (long *) &((GetInfoRequestBlockPtr) (paramPtr)->requestIOBlock)->dcmdVersion = (version)
#define dcmdFillString(paramPtr, which, theStr) \
	BlockMoveData(theStr, &((GetInfoRequestBlockPtr) paramPtr->requestIOBlock)->which, (theStr)[0]+1)

/*
	MacsBug callback routines that can be called by the dcmd.
*/

/*
	Draw the text in the Pascal string as one or more lines separated by CR's.
	Each line causes the MacsBug display to be scrolled and the new line to be
	drawn at the bottom. If the user types a key while scrolling then the aborted
	flag is set telling the command to terminate immediately.
*/
extern pascal void dcmdDrawLine(ConstStr255Param str);

/*
	Draw the text in the Pascal string as a continuation of the current line.
	CR's are not given special treatment.
*/
extern pascal void dcmdDrawString(ConstStr255Param str);

/*
	Draw a given number of characters starting from the given pointer as a 
	continuation of the current line. CR's are not given special treatment.
*/
extern pascal void dcmdDrawText(StringPtr text, short length);

/*
	Scrolls the MacsBug display up one line leaving a blank line at the bottom.
*/
extern pascal void dcmdScroll(void );

/*
	Display the Pascal string in the command line area and wait for a key to be pressed.
	Return TRUE if the user typed Return. All other keys return FALSE. MacsBug saves this
	key and adds it to the command line once the current command completes. Typing any
	key other than Return sets the aborted flag and tells the command to terminate immediately.
*/
extern pascal Boolean dcmdDrawPrompt(ConstStr255Param str);

/*
	Get the current command line position.
*/
extern pascal short dcmdGetPosition(void );

/*
	Set the current command line position. This should only be set to a value returned 
	by dcmdGetPosition.
*/
extern pascal void dcmdSetPosition(short pos);

/*
	Return the next character on the command line or CR if the entire line has been scanned
*/
extern pascal short dcmdGetNextChar(void );

/*
	Return the next character on the command line or CR if the entire line has been scanned.
	However, the current command line position is not changed.
*/
extern pascal short dcmdPeekAtNextChar(void );

/*
	Copy all characters from the command line to the parameter string until a delimiter
	is found or the end of the command line is reached. A delimiter is either a space,
	a comma or a CR. Both single and double quoted strings are allowed on the command
	line. However, the leading and trailing quotes must be of the same type. The parameter
	string is returned without the quotes. This function returns the delimiter.
*/
extern pascal short dcmdGetNextParameter(Str255 str);

/*
	Parse the command line for the next expression. All expressions are evaluated to 32 bits.
	This function returns the delimiter after the expression. The possible delimiters are
	space, comma and CR. Space is not treated as a delimiter in the middle of expressions,
	unless it's before a binary '-' (minus). For instance, '1 + 2' will return a value of 3
	and the delimiter will be the char following the 2. '1 - 2' will return a value of 1
	and the delimeter is the space character before the '-'. This is done so that built-in
	commands and dcmds can properly parse '-x' options. The return parameter 'ok' tells if
	the expression was parsed successfully as far as syntax is concerned.
*/
extern pascal short dcmdGetNextExpression(long *value, Boolean *ok);

/*
	Copy the break message MacsBug displayed the last time it was entered into str.
	This may contain multiple lines separated by CR's.
*/
extern pascal void dcmdGetBreakMessage(Str255 str);

/*
	Return a symbolic representation for address in str. If no symbol can be found
	then an empty string is returned. The format of the symbol returned is Name+00000.
*/
extern pascal void dcmdGetNameAndOffset(long address, Str255 str);

/*
	Return the trap name for the trap number. If no symbol can be found
	then an empty string is returned.
*/
extern pascal void dcmdGetTrapName(short trapNumber, Str255 trapName);

/*
	Return a pointer the macro name for the given value. If no macro can be found
	then a nil is returned.
*/
extern pascal StringPtr dcmdGetMacroName(long value);

/*
	When a debugger command is called, the debugger's world (low memory) is installed.
	Commands that want to reference the user's world can swap back and forth between the
	two worlds by making this call. This procedure does nothing in MacsBug; there is no
	distinction between user and debugger worlds. It is included to support other
	debuggers that might want to take advantage of it.
*/
extern pascal void dcmdSwapWorlds(void );

/*
	Toggle between the user and debugger displays. The first call restores the user's actual screen.
	The second call restores the debugger's screen.
*/
extern pascal void dcmdSwapScreens(void );

/*
	Walk through the blocks in the current heap, calling DoThis for each block. DoThis should
	be a procedure of the form:
				
		pascal void DoThis (long blockAddress, long blockLength, long addrOfMasterPtr,
			 				short blockType, Boolean locked, Boolean purgeable, Boolean resource)

	The blockAddress and blockLength pertain to the data in the heap block, not including
	the block header. The addrOfMasterPtr is the master pointer's location in the heap,
	not the value of the master ptr. The blockType is defined by the constants freeBlock,
	nonrelocatableBlock and relocatableBlock. The booleans locked, purgeable and resource
	reflect the state of the block.
*/

typedef pascal void (*DoThisPtr)(long blockAddress, long blockLength, long addrOfMasterPtr, short blockType, Boolean locked, Boolean purgeable, Boolean resource);

extern pascal void dcmdForAllHeapBlocks(DoThisPtr DoThis);

#if PRAGMA_IMPORT_SUPPORTED
#pragma import off
#endif

#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=reset
#endif

#ifdef __cplusplus
}
#endif

#endif /* __DCMD__ */

